#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import apt
import apt.progress.base as apb
from apt.debfile import DebPackage
from apt.cache import FetchFailedException
import locale


class AppActions:
    INS_AND_UP = "install_and_upgrade"
    INSTALL = "install"
    INSTALL_ONE = "insone"
    UPGRADE = "upgrade"
    UPGRADE_ONE = "upone"
    UPDATE = "update"
    UPDATESHELL = "update_by_shell"
    CONFIGURESHELL = "configure_dpkg_by_shell"


class FetchProcess(apb.AcquireProgress):

    def __init__(self, dbus_service, appname, action):
        apb.AcquireProgress.__init__(self)
        self.dbus_service = dbus_service
        self.appname = appname
        self.action = action

    def done(self, item):
        kwarg = {"download_appname":self.appname,
                 "download_percent":str(self.percent),
                 "action":str(self.action),
                 }

        self.dbus_service.kum_fetch_signal("down_done", kwarg)

    def fail(self, item):
        kwarg = {"download_appname":self.appname,
                 "download_percent":str(self.percent),
                 "action":str(self.action),
                 }

        self.dbus_service.kum_fetch_signal("down_fail", kwarg)

    def fetch(self, item):
        kwarg = {"download_appname":self.appname,
                 "download_percent":str(self.percent),
                 "action":str(self.action),
                 }

        self.dbus_service.kum_fetch_signal("down_fetch", kwarg)

    def ims_hit(self, item):
        #print 'ims_hit'
        pass

    def media_change(self, media, drive):
        #print 'media_change'
        pass

    def pulse(self, owner):

        if self.action == AppActions.UPDATE:
            if self.total_items!= 0:
                percent = float(self.current_items * 100.0 / self.total_items)
                if percent > self.percent:
                    self.percent = percent
        else:
            if self.total_bytes != 0:
                self.percent = float(self.current_bytes * 100.0 / self.total_bytes)

        kwarg = {"download_appname":self.appname,
                 "download_percent":str(self.percent),
                 "download_bytes":str(self.current_bytes),
                 "total_bytes":str(self.total_bytes),
                 "download_items":str(self.current_items),
                 "total_items":str(self.total_items),
                 "action":str(self.action),
                 }

        self.dbus_service.kum_fetch_signal("down_pulse", kwarg)

        # cancel the operation
        if self.dbus_service.check_cancel_worker_item(self.appname) is True:
            print "download_cancel"
            self.dbus_service.kum_fetch_signal("down_cancel", kwarg)
            return False

    def start(self):
        # Reset all our values.
        self.current_bytes = 0.0
        self.current_cps = 0.0
        self.current_items = 0
        self.elapsed_time = 0
        self.fetched_bytes = 0.0
        self.last_bytes = 0.0
        self.total_bytes = 0.0
        self.total_items = 0
        self.percent = 0

        kwarg = {"download_appname":self.appname,
                 "download_percent":str(self.percent),
                 "action":str(self.action),
                 }

        self.dbus_service.kum_fetch_signal("down_start", kwarg)

    def stop(self):
        kwarg = {"download_appname":self.appname,
                 "download_percent":str(200),
                 "action":str(self.action),
                 }

        self.dbus_service.kum_fetch_signal("down_stop", kwarg)


class AptProcess(apb.InstallProgress):
    '''Apt progress'''
    def __init__(self, dbus_service, appname, action):
        apb.InstallProgress.__init__(self)
        self.dbus_service = dbus_service
        self.appname = appname
        self.percent = 0
        self.action = action

    def conffile(self, current, new):
        pass

    def error(self, pkg, errormsg):
        kwarg = {"apt_appname":str(pkg),
                 "apt_percent":str(self.percent),
                 "action":str(self.action),
                 }
        self.dbus_service.kum_apt_signal("apt_error", kwarg)

    def start_update(self):
        kwarg = {"apt_appname":self.appname,
                 "apt_percent":str(self.percent),
                 "action":str(self.action),
                 }

        self.dbus_service.kum_apt_signal("apt_start", kwarg)

    def finish_update(self):
        kwarg = {"apt_appname":self.appname,
                 "apt_percent":str(200),
                 "action":str(self.action),
                 }

        self.dbus_service.kum_apt_signal("apt_finish", kwarg)

    def status_change(self, pkg, percent, status):
        kwarg = {"apt_appname":str(pkg),
                 "apt_percent":str(percent),
                 "action":str(self.action),
                 "status":str(status),
                 }

        self.dbus_service.kum_apt_signal("apt_pulse", kwarg)


class AptHandler():

    def __init__(self, dbus_service):
        self.dbus_service = dbus_service
        locale.setlocale(locale.LC_ALL, "zh_CN.UTF-8")
        self.cache = apt.Cache()
        self.cache.open()

    def get_pkg_by_name(self, pkgName):
        try:
            return self.cache[pkgName]
        except KeyError:
            raise WorkitemError(1, "Package %s is not  available" % pkgName)

    # 批量自动安装或升级
    def install_and_upgrade(self, pkg_names, kwargs=None):
        self.cache.open()

        for pkg_name in pkg_names:
            pkg = self.get_pkg_by_name(pkg_name)

            if(pkg.is_installed == False):
                pkg.mark_install()
            if(pkg.is_upgradable == True):
                pkg.mark_upgrade()

        try:
            self.cache.commit(FetchProcess(self.dbus_service, "#install_and_upgrade", AppActions.INS_AND_UP), AptProcess(self.dbus_service, "#install_and_upgrade", AppActions.INS_AND_UP))
        except apt.cache.FetchFailedException as error:
            raise WorkitemError(2, str(error))
        except apt.cache.LockFailedException:
            raise WorkitemError(3, "package manager is running.")
        except Exception as e:
            raise WorkitemError(0, "unknown error")

    # 批量安装
    def install(self, pkgNames, kwargs=None):
        self.cache.open()

        for pkgName in pkgNames:
            pkg = self.get_pkg_by_name(pkgName)
            pkg.mark_install()

        try:
            self.cache.commit(FetchProcess(self.dbus_service, "#install", AppActions.INSTALL), AptProcess(self.dbus_service, "#install", AppActions.INSTALL))
        except apt.cache.FetchFailedException as error:
            raise WorkitemError(2, str(error))
        except apt.cache.LockFailedException:
            raise WorkitemError(3, "package manager is running.")
        except Exception as e:
            raise WorkitemError(0, "unknown error")

    # 单个安装
    def insone(self, pkgName, kwargs=None):
        self.cache.open()
        pkg = self.get_pkg_by_name(pkgName)
        pkg.mark_install()

        try:
            self.cache.commit(FetchProcess(self.dbus_service, pkgName, AppActions.INSTALL_ONE), AptProcess(self.dbus_service, pkgName, AppActions.INSTALL_ONE))
        except apt.cache.FetchFailedException as error:
            raise WorkitemError(2, str(error))
        except apt.cache.LockFailedException:
            raise WorkitemError(3, "package manager is running.")
        except Exception as e:
            raise WorkitemError(0, "unknown error")

    # 批量升级
    def upgrade(self, pkgNames, kwargs=None):
        self.cache.open()

        for pkgName in pkgNames:
            pkg = self.get_pkg_by_name(pkgName)
            pkg.mark_upgrade()

        try:
            self.cache.commit(FetchProcess(self.dbus_service, "#upgrade", AppActions.UPGRADE), AptProcess(self.dbus_service, "#upgrade", AppActions.UPGRADE))
        except apt.cache.FetchFailedException as error:
            raise WorkitemError(2, str(error))
        except apt.cache.LockFailedException:
            raise WorkitemError(3, "package manager is running.")
        except Exception as e:
            raise WorkitemError(0, "unknown error")

    # 单个升级
    def upone(self, pkgName, kwargs=None):
        self.cache.open()
        pkg = self.get_pkg_by_name(pkgName)
        pkg.mark_upgrade()

        try:
            self.cache.commit(FetchProcess(self.dbus_service, pkgName, AppActions.UPGRADE_ONE), AptProcess(self.dbus_service, pkgName, AppActions.UPGRADE_ONE))
        except apt.cache.FetchFailedException as error:
            raise WorkitemError(2, str(error))
        except apt.cache.LockFailedException:
            raise WorkitemError(3, "package manager is running.")
        except Exception as e:
            raise WorkitemError(0, "unknown error")

    # 调用python-apt库apt-get update
    def update(self, taskName, kwargs=None):
        quiet = False
        if kwargs is not None:
            quiet = int(kwargs["quiet"])

        try:
            if quiet == True:
                print "quiet=True"
                self.cache.update()
            else:
                print "quiet=False"
                self.cache.update(fetch_progress=FetchProcess(self.dbus_service, taskName, AppActions.UPDATE))
        except Exception, e:
            print e
            print "update except"

    # 调用本地命令apt-get update
    def update_by_shell(self, taskName, kwargs=None):
        out_msg = os.popen("apt-get update").readlines()

        kwarg = {"apt_appname":str(taskName),
                 "apt_percent":str(200),
                 "action":"#update_by_shell",
                 "status":str("ha"),
                 }

        self.dbus_service.kum_apt_signal("finish", kwarg)

    # 调用本地命令修复dpkg
    def configure_dpkg_by_shell(self, taskName, kwargs=None):
        out_msg = os.popen("dpkg --configure -a").readlines()

        kwarg = {"apt_appname":str(taskName),
                 "apt_percent":str(200),
                 "action":"#configure_dpkg_by_shell",
                 "status":str("ha"),
                 }

        self.dbus_service.kum_apt_signal("finish", kwarg)


class WorkitemError(Exception):

    def __init__(self, errornum, details = ""):
        self.errornum = errornum
        self.details = details


if __name__ == "__main__":
    ad = AptHandler(None)
    ad.upgrade(['khelpcenter4'])
